#include <unistd.h>
#include <fcntl.h>
#include "globdef.h"
#include "uidef.h"
#include "sdrdef.h"
#include "conf.h"
#include "fft1def.h"
#include "screendef.h"
#include "thrdef.h"

#define MAX_LIBUSB_BULK_BUFFER 16320


#if PERSEUSLIB_PRESENT == 1
#include <perseus-sdr.h>
perseus_descr *descr;
#endif

#define SDR14_DEVICE_DRIVER "/dev/linrad_ft245"
// Perseus receiver USB buffering scheme definitions (never touch)
#define PERSEUS_SAMPLESPERFRAME	170   // Samples in one FX2 USB transaction
#define PERSEUS_COMPONENTSPERSAMPLE 2 // I & Q components
#define PERSEUS_BYTESPERCOMPONENT   3 // 24 bit data
// Derived defines
#define PERSEUS_BYTESPERSAMPLE (PERSEUS_BYTESPERCOMPONENT*PERSEUS_COMPONENTSPERSAMPLE)
#define PERSEUS_BYTESPERFRAME (PERSEUS_SAMPLESPERFRAME*PERSEUS_BYTESPERSAMPLE)

extern P_PERSEUS pers;
extern float perseus_filter_cutoff[MAX_PERSEUS_FILTER+1];
extern char *perseus_bitstream_names[MAX_PERSEUS_RATES];
int perseus_bytesperbuf;
int timf1p_pers;


void open_excalibur(void)
{
}


void set_perseus_frequency(void)
{
#if PERSEUSLIB_PRESENT == 1
double dt1;
dt1=fg.passband_center*1000000+0.5;
if(dt1 < 0 || dt1 > adjusted_sdr_clock/2)
  {
  fg.passband_center=7.05;
  dt1=fg.passband_center*1000000;
  }
perseus_set_ddc_center_freq(descr, dt1, pers.presel);
#endif
}

void fill_perseus_rates(void)
{
#if PERSEUSLIB_PRESENT == 1
if (perseus_get_sampling_rates (descr, perseus_rates, PERSEUS_RATES_BUFSIZE))
  {
  lirerr(1196);
  return;
  }
no_of_perseus_rates = 0;
while (perseus_rates[no_of_perseus_rates] > 1) 
  {
  no_of_perseus_rates++;
  }
#endif
}

void perseus_stop(void)
{
#if PERSEUSLIB_PRESENT == 1
perseus_stop_async_input(descr);
lir_sleep(10000);
#endif
}



int perseus_callback(void *buf, int buf_size, void *extra)
{
int i, j;
register char c1;
char *pBuf;
pBuf=extra;
pBuf=(void*)buf;
c1=0;
// This callback gets called whenever a buffer is ready
// from the USB interface
j=timf1p_pers;
for(i=0; i<buf_size; i+=6)
  {
  timf1_char[timf1p_pers+3]=pBuf[i+2];
  c1=pBuf[i+2]&0x80;
  timf1_char[timf1p_pers+2]=pBuf[i+1];
  timf1_char[timf1p_pers+1]=pBuf[i  ];
  c1|=pBuf[i];
  timf1_char[timf1p_pers  ]=0;
  timf1_char[timf1p_pers+7]=pBuf[i+5];
  timf1_char[timf1p_pers+6]=pBuf[i+4];
  timf1_char[timf1p_pers+5]=pBuf[i+3];
  timf1_char[timf1p_pers+4]=0;
  timf1p_pers=(timf1p_pers+8)&timf1_bytemask;
  }
if( (c1&1) != 0)
  {
  ad_maxamp[0]=0x7fff;
  ad_maxamp[1]=0x7fff;
  }
lir_set_event(EVENT_PERSEUS_INPUT);
return 0;
}

void start_perseus_read(void)
{
#if PERSEUSLIB_PRESENT == 1
int i;
sdr=-1;
perseus_bytesperbuf=snd[RXAD].block_bytes/PERSEUS_BYTESPERFRAME;
perseus_bytesperbuf*=PERSEUS_BYTESPERFRAME;
if(perseus_bytesperbuf > MAX_LIBUSB_BULK_BUFFER)
  {
  perseus_bytesperbuf=MAX_LIBUSB_BULK_BUFFER/PERSEUS_BYTESPERFRAME;
  perseus_bytesperbuf*=PERSEUS_BYTESPERFRAME;
  while(snd[RXAD].block_bytes > perseus_bytesperbuf)
    {
    snd[RXAD].block_bytes/=2;
    snd[RXAD].block_frames/=2;
    snd[RXAD].interrupt_rate*=2;
    }
  }
  
  
  
  
timf1p_pers=timf1p_pa;
lir_sched_yield();
i=perseus_start_async_input(descr,perseus_bytesperbuf, perseus_callback, NULL);
if(i < 0) 
  {
  lirerr(1195);
  return;
  }
sdr=0;
#endif
}

void lir_perseus_read(void)
{
if(((timf1p_pers-timf1p_pa+timf1_bytes)&timf1_bytemask) < (int)snd[RXAD].block_bytes)
  {
  lir_await_event(EVENT_PERSEUS_INPUT);
  }
timf1p_pa=(timf1p_pa+snd[RXAD].block_bytes)&timf1_bytemask;
}

int perseus_load_proprietary_code(int rate_no)
{
#if PERSEUSLIB_PRESENT == 1
int i;
i=perseus_set_sampling_rate_n(descr, rate_no);
if(i<0) return 1194;
return 0;
#else
return rate_no;
#endif
}

int perseus_eeprom_read(unsigned char *buf, int addr, unsigned char count)
{
#if PERSEUSLIB_PRESENT == 1
eeprom_prodid *prodid;
int i;
char x;
char s[80];
prodid=(void*) buf;
x=buf[0];
i=addr;
x=count;
i=perseus_get_product_id(descr,prodid);
if(i<0)
  {
  sprintf(s,"get product id error: %s", perseus_errorstr());
  clear_screen();
  lir_text(5,5,s);
  lir_text(5,7,press_any_key);
  await_processed_keyboard();
  lirerr(1193);
  return 1;
  }
return 0;
#else
buf[0]=addr;
return count;
#endif
}

void open_perseus(void)
{
#if PERSEUSLIB_PRESENT == 1
int i;
//perseus_set_debug(3);
if(sdr != -1)return;
if(perseus_init() == 0)return;
descr=perseus_open(0);
if(descr == NULL)return;
i=perseus_firmware_download(descr,NULL);
if(i < 0)
  {
  close_perseus();
  return;
  }
sdr=0;  
#endif
}

void perseus_store_att(unsigned char idAtt)
{
#if PERSEUSLIB_PRESENT == 1
perseus_set_attenuator(descr, idAtt);
#else
(void)idAtt;
#endif
}

void perseus_store_presel(unsigned char idPresel)
{
char x;
x=idPresel;
}

void close_perseus(void)
{
#if PERSEUSLIB_PRESENT == 1
sdr=-1;
perseus_close(descr);
perseus_exit();
#endif
}

int zrwx;
void lir_sdr14_write(void *s, int bytes)
{
zrwx=write(sdr,s,bytes);
}

int lir_sdr14_read(void *s,int bytes)
{
int i;
i=read(sdr, s, bytes);
return i;
}

void open_sdr14(void)
{
SNDLOG"Trying to open %s\n",SDR14_DEVICE_DRIVER);
sdr=open( SDR14_DEVICE_DRIVER ,O_RDWR );
}

void close_sdr14(void)
{
int i;
sdr=-1;
i=fcntl(sdr, F_GETFD);
i|=O_NONBLOCK;
fcntl(sdr,F_SETFD,i);
lir_sleep(10000);
close(sdr);
}

